function [Pgen,Pd,Pcp,Pcm,Pgp,Pgm,Esoc] =...
           run_alg3(Pgen0,Pload0,base_params,B_params,co2,Crate)
% [Pgen,Pd,Pcp,Pcm,Pgp,Pgm,Esoc] =...
%            run_alg3(Pgen0,Pload0,base_params,B_params,co2)
%     runs Emissions Arbitrage Algorithm using parameters input. Pgen0 =
%     normalised PV generation series, Pload0 = electricity consumption
%     series, base_params = system parameters, B_params = battery
%     parameters, co2 = MEF(t), Crate = battery power C-rate limit. Outputs
%     [Pgen,Pd,Pcp,Pcm,Pgp,Pgm,Esoc] - time series of PV, load, charging
%     power, discharging power, export power, import power, energy stored.
%% unpack base_params
  P_PV = base_params{1};
  E_B = base_params{2};
%   lambda_1 = base_params{3};
%   lambda_2 = base_params{4};
  dt = base_params{3};
  t = base_params{4};
  Ndt = length(t);
  Pgen = P_PV*Pgen0;
  Pd = Pload0;
%   chi_p = (1-2*lambda_2)./(1-lambda_1-lambda_2);
%   chi_m = 1./(1-lambda_1-lambda_2);
  eta_dc = B_params{1}; % DC-DC converter efficiency (1-way)
  eta_inv = B_params{2}; % inverter efficiency (1-way)
  eta_B = B_params{3}; %[begin end] charge efficiency
  E_B_low = B_params{4}*E_B; % kWh low limit
  E_B_high = B_params{5}*E_B; % kWh high limit
  P_B = Crate*E_B; % max charge/discharge C-rate
  selfdis = B_params{7};
  
%% run the damn sim
  Pgp = zeros(1,Ndt); % grid export time series, kW
  Pgm = zeros(1,Ndt); % grid import time series, kW
  Pgm(1) = Pd(1); % initialise at step-0 grid import
  Pcp = zeros(1,Ndt); % battery charging power, kW
  Pcm = zeros(1,Ndt); % battery discharging power, kW
  Esoc = ones(1,Ndt)*E_B_low; % battery stored energy, kWh
  
  co2smd = smooth([co2,co2,co2],ceil(30*24/dt)); % smoothed CO2 profile
  % built up from past 30 days' data
  co2smd = co2smd(Ndt-floor(30*24/dt):2*Ndt-floor(30*24/dt)-1);
  limhi = co2smd+0.02; % if CO2 > limhi, then discharge
  limlo = co2smd-0.02; % if CO2 < limlo, then charge
  co2 = circshift(co2,round(1/12/dt)); % delay of 5min to calc correct CO2
    
  for nt = 2:Ndt
   % battery operation determined by grid CO_2 intensity   
   if co2(nt) > limhi(nt) % if CO_2 intensity high, discharge battery
    Pcm(nt) = min([P_B,(Esoc(nt-1)-E_B_low)*eta_dc*eta_B/dt]);
    % import remainder from grid:
    Pgm(nt) = max([0,Pd(nt)-eta_inv*(Pgen(nt)+Pcm(nt))]);
    % or export remainder to grid:
    Pgp(nt) = max([0,(Pgen(nt)+Pcm(nt))*eta_inv-Pd(nt)]);
    % update battery stored energy:
    Esoc(nt) = Esoc(nt-1) - dt*Pcm(nt)/(eta_B*eta_dc);
   elseif co2(nt) < limlo(nt) % if CO_2 intensity low, charge battery 
    Pcp(nt) = min([P_B,(E_B_high-Esoc(nt-1))/(dt*eta_B*eta_dc)]);
    % export remainder to grid:
    Pgp(nt) = max([0,(Pgen(nt)-Pcp(nt))*eta_inv-Pd(nt)]);
    % or import remainder from grid:
    if (Pgen(nt)-Pcp(nt)) >= 0
      Pgm(nt) = max([0,Pd(nt)-eta_inv*(Pgen(nt)-Pcp(nt))]);
    else % (Pgen(nt)-Pcp(nt)) < 0
      Pgm(nt) = max([0,Pd(nt)-(Pgen(nt)-Pcp(nt))/eta_inv]);
    end
    % update battery stored energy:
    Esoc(nt) = Esoc(nt-1) + dt*Pcp(nt)*eta_B*eta_dc;
   else
    % import remainder from grid:
    Pgm(nt) = max([0,Pd(nt)-eta_inv*Pgen(nt)]);
    % or export remainder to grid:
    Pgp(nt) = max([0,Pgen(nt)*eta_inv-Pd(nt)]);  
    % update battery stored energy:
    Esoc(nt) = Esoc(nt-1);
   end
   Esoc(nt) = Esoc(nt) - selfdis*E_B*dt;
  end

end

